﻿/*
	This script is intended to dynamically control your rover's wheels' max steering angle according to your current speed.
	It contains several driving profiles and allows you enable/disable features like downforce thrusters or rear wheels steering, by selecting appropriate profile.
	Instructions:
	
	1. You will need one programmable block, one timer, at least one block group with wheels (ONLY WHEELS, nothing else!).
	1a. Optional: Another group with wheels (ONLY WHEELS, nothing else!), which will be treated as rear wheels and controlled accordingly.
	1b. Optional: Group with OVERRIDED Thrusters (ONLY THRUSTERS, nothing else!), pointed downwards for extra downforce. They will be enabled if Race profile is active.
	1c. Optional: One LCD/Text panel that will display your rover status.
	2. Set the timer to run this script with empty Argument. Also set the time to START itself and to TRIGGER_NOW itself. You need this scrip restarted really fast. Do not start the timer yet!
	3. Go to basic options below and set them according to your preferences.
	3a. Optional: Adjust advanced options below.
	4. Set several actions at your cockpit:
		Run this script with "Parking" (no "" needed) argument. Case insensitive. - It will set your rover to "Parking" mode.
		Run this script with "Slow" (no "" needed) argument. Case insensitive. - It will set your rover to "Slow" mode.
		Run this script with "Normal" (no "" needed) argument. Case insensitive. - It will set your rover to "Normal" mode.
		Run this script with "Race" (no "" needed) argument. Case insensitive. - It will set your rover to "Race" mode. It's the only mode with downforce thrusters enabled!
	4a. Optional: Instead of step 4 (or even with it) set 2 actions at your cockpit:
		Run this script with "GearUp" (no "" needed) argument. Case insensitive. - It will "shift the gear Up" and switch your rover to the Next mode, like from Parking to Slow, from Slow to Normal and so on.
		Run this script with "GearDown" (no "" needed) argument. Case insensitive. - As previous one but vice versa.
	5. Trigger your timer, so the script will start.
	6. Drive around to check if you like the behaviour of the rover and adjust options if needed.
	
	Basic options are mandatory, make sure to set them properly before using the script!

	Basic options beyond this point. You PROBABLY want to edit them. Advanced options are further below.
=====================================================================================================
*/

//Block and group naming
string GroupName = "Wheels - Steering"; //Your lead wheels GROUP name. Must be EXACT NAME of the group. Case sensitive.
string GroupNameBack = "Wheels - Rear"; //Your rear wheels GROUP name. Must be EXACT NAME of the group. Case sensitive. Optional if RearSteer set to false.
string GroupNameAll = "Wheels - All"; //Your GROUP, that contain ALL rover's wheels. Must be EXACT NAME of the group. Case sensitive. Mandatory.
string ThrustersDownName = "Thrusters - Down"; //Your OVERRIDED thrusters for additional downforce GROUP name. Must be EXACT NAME of the group. Case sensitive. Optional if DownforceThrusters set to false.
string LCDNamePrefix = "[LCD Steer]"; //Your information display name PREFIX. Actual name should START with it. Case sensitive. Optional if UseLCD set to false.

//Custom GROUPS of blocks, that will be enabled/disabled when you change drive modes. To disable group usage: set the group name to "" (quotes with nothing between them. Not even a space! Nothing.)
string GroupParkingName = ""; //Your block GROUP to be enabled when you activate parking mode. Will be DISABLED when you select another drive mode!
string GroupSlowName = ""; //Your block GROUP to be enabled when you activate slow mode. Will be DISABLED when you select another drive mode!
string GroupNormalName = ""; //Your block GROUP to be enabled when you activate normal mode. Will be DISABLED when you select another drive mode!
string GroupRaceName = ""; //Your block GROUP to be enabled when you activate race mode. Will be DISABLED when you select another drive mode!

//General options
bool RearSteer = true; //Set to true to enable rear wheels steering in parking mode.
bool RearSteerOnSlow = false; //Set to true if you want your rear wheels to have steering enabled in Slow mode.
bool DynamicRearSteer = true; //Set to true if you want script to control rear wheels steering angle dynamically. Will only work if previous set to true! Rear steering will still be disabled in Race mode!
bool DownforceThrusters = true; //Determines if the script will try to use thrusters for additional downforce.
bool UseLCD = true; //Determines if the script will use LCD or Text screen to provide you with it's current status.

//Speed settings for different drive modes
bool IgnoreSpeed = false; //If set to true script will not manage max speed of your rover. 
string MaxSpeedParking = "12"; //Max speed in kilometers per hour for parking mode. Should be low.
string MaxSpeedSlow = "30"; //Max speed in kilometers per hour for slow mode.
string MaxSpeedNormal = "100"; //Max speed in kilometers per hour for normal mode.
string MaxSpeedRace = "150"; //Max speed in kilometers per hour for race mode. Set to 360 for UNLIMITED.

//Steering angle presets editing is optional.
double AngleMax = 30; //Maximum steering angle for your rover.
double AngleMin = 7; //Minumum normal steering angle for your rover.
double AngleSuperMin = 4; //Super minimum angle. Intended for hight speed travel only.

//Very same for rear wheels. Optional.
double RAngleMax = 15; //Maximum steering angle for your rover's rear wheels. Will only work in certain drive modes and only if rear steering enabled.
double RAngleMin = 5; //Minumum normal steering angle for your rover's rear wheels. Will only work in certain drive modes and only if rear steering enabled.
double RAngleSuperMin = 0; //Super minimum angle. Intended for hight speed travel only. Set to 0 by default, because rear steering at hight speed is SUPER UNSAFE! You can set it above 0 if you dare.

//Speed presets editing is optional
double VelocityMin = 20; //Below this speed your steering angle will be maxed
double VelocityMax = 60; //Above this speed your steering angle will be locked to minimum
double VelocitySuperMax = 100; //Above this speed your steering angle will be locked to super minumum.




/*
	Advanced options beyond this point. Settings below will allow you to adjust your drivetrain according to driving profile.
===================================================================================================================
*/

//Wheels settings for different modes. Everything ignored by default.
bool IgnorePower = false; //If the script should ignore power settings. It will not alter your wheels power settings if set to true.
string PowerParking = "70"; //Wheels power to be used in Parking mode. Between "0" and "100".
string PowerSlow = "20"; //Wheels power to be used in Slow mode. Between "0" and "100".
string PowerNormal = "30"; //Wheels power to be used in Normal mode. Between "0" and "100".
string PowerRace = "40"; //Wheels power to be used in Race mode. Between "0" and "100".
bool IgnoreFriction = true; //If the script should ignore friction settings. It will not alter your wheels friction settings if set to true.
string FrictionParking = "80"; //Wheels friction setting to be used in Parking mode. Between "0" and "100".
string FrictionSlow = "70"; //Wheels friction setting to be used in Slow mode. Between "0" and "100".
string FrictionNormal = "60"; //Wheels friction setting to be used in Normal mode. Between "0" and "100".
string FrictionRace = "60"; //Wheels friction setting to be used in Race mode. Between "0" and "100".
bool IgnoreStrength = false; //If the script should ignore strength settings. It will not alter your wheels strength settings if set to true.
string StrengthParking = "17"; //Wheels strength setting to be used in Parking mode. Between "0" and "100".
string StrengthSlow = "25"; //Wheels strength setting to be used in Slow mode. Between "0" and "100".
string StrengthNormal = "20"; //Wheels strength setting to be used in Normal mode. Between "0" and "100".
string StrengthRace = "25"; //Wheels strength setting to be used in Race mode. Between "0" and "100".
bool IgnoreHeight = true; //If the script should ignore height settings. It will not alter your wheels height settings if set to true.
string HeightParking = "-0.32"; //Drive height setting to be used in Parking mode. Check your wheels to find min and max.
string HeightSlow = "-0.32"; //Drive height setting to be used in Slow mode. Check your wheels to find min and max.
string HeightNormal = "-0.32"; //Drive height setting to be used in Normal mode. Check your wheels to find min and max.
string HeightRace = "-0.32"; //Drive height setting to be used in Race mode. Check your wheels to find min and max.
//Following settings were removed from wheels suspensions block. They are still here just because I'm yet to properly clear them out. Changing them will do nothing.
bool IgnoreTravel = true; //If the script should ignore travel settings. It will not alter your wheels travel settings if set to true.
string TravelParking = "90"; //Wheels suspension travel setting to be used in Parking mode. Between "0" and "100".
string TravelSlow = "90"; //Wheels suspension travel setting to be used in Slow mode. Between "0" and "100".
string TravelNormal = "90"; //Wheels suspension travel setting to be used in Normal mode. Between "0" and "100".
string TravelRace = "90"; //Wheels suspension travel setting to be used in Race mode. Between "0" and "100".
bool IgnoreDamping = true; //If the script should ignore damping settings. It will not alter your wheels damping settings if set to true. 
string DampingParking = "30"; //Wheels damping setting to be used in Parking mode. Between "0" and "100".
string DampingSlow = "50"; //Wheels damping setting to be used in Slow mode. Between "0" and "100".
string DampingNormal = "80"; //Wheels damping setting to be used in Normal mode. Between "0" and "100".
string DampingRace = "80"; //Wheels damping setting to be used in Race mode. Between "0" and "100".

/*
	Main script starts here. Better not to edit, unless you are sure and know what you are doing!
=====================================================================================================
=====================================================================================================
=====================================================================================================
*/

//Variable declarations beyond this point.
bool debug = false;
string Angle;
string RAngle;
string Shift;
int Gear;
List<IMyBlockGroup> BlockGroups = new List<IMyBlockGroup>{};
List<IMyTerminalBlock> WheelsMain = new List<IMyTerminalBlock>{};
List<IMyTerminalBlock> GroupBack = new List<IMyTerminalBlock>{};
List<IMyTerminalBlock> GroupAll = new List<IMyTerminalBlock>{};
List<IMyTerminalBlock> ThrustersDown = new List<IMyTerminalBlock>{};

List<IMyTerminalBlock> GroupParking = new List<IMyTerminalBlock>{};
List<IMyTerminalBlock> GroupSlow = new List<IMyTerminalBlock>{};
List<IMyTerminalBlock> GroupNormal = new List<IMyTerminalBlock>{};
List<IMyTerminalBlock> GroupRace = new List<IMyTerminalBlock>{};
List<IMyTerminalBlock> CustomGroupsAll = new List<IMyTerminalBlock>{};


double Velocity;
bool hasRun = false;
Vector3D previousPos;   
Vector3D currentPos;   
double previousVelocity;   
double currentVelocity;
double delta_distance;
int tick_limit = 10;
int current_tick = 1;



void Main(string arg){
	arg = arg.ToLower();
	if(current_tick % tick_limit == 0)
    {
		
		if (debug) Echo("Storage at start: "+Storage);
		GetGroups();
		PosVelAccel();

		//Selecting proper steering angle
		/*
		if (Velocity <= VelocityMin){
			Angle = (AngleMax/60).ToString();
			RAngle = (RAngleMax/60).ToString();
		} else if (Velocity >= VelocityMax && Velocity < VelocitySuperMax) {
			Angle = (AngleMin/60).ToString();
			RAngle = (RAngleMin/60).ToString();
		} else if (Velocity >= VelocitySuperMax) {
			Angle = (AngleSuperMin/60).ToString();
			RAngle = (RAngleSuperMin/60).ToString();
		} else {
			double Temp1 = VelocityMax-Velocity;
			double Temp2 = VelocityMax - VelocityMin;
			double Temp3 = Temp2/100;
			double Temp = Temp1/Temp3;
			Angle = (((((AngleMax - AngleMin)/100)*Temp)+AngleMin)/60).ToString();
			RAngle = (((((RAngleMax - RAngleMin)/100)*Temp)+RAngleMin)/60).ToString();			
		}
		*/
		if (Velocity <= VelocityMin){
			Angle = (AngleMax).ToString();
			RAngle = (RAngleMax).ToString();
		} else if (Velocity >= VelocityMax && Velocity < VelocitySuperMax) {
			Angle = (AngleMin).ToString();
			RAngle = (RAngleMin).ToString();
		} else if (Velocity >= VelocitySuperMax) {
			Angle = (AngleSuperMin).ToString();
			RAngle = (RAngleSuperMin).ToString();
		} else {
			double Temp1 = VelocityMax-Velocity;
			double Temp2 = VelocityMax - VelocityMin;
			double Temp3 = Temp2/100;
			double Temp = Temp1/Temp3;
			Angle = (((((AngleMax - AngleMin)/100)*Temp)+AngleMin)).ToString();
			RAngle = (((((RAngleMax - RAngleMin)/100)*Temp)+RAngleMin)).ToString();			
		}		
		if (debug) Echo("Angle value: "+Angle);
		
		//"Gears" control
		switch (Storage) {
			case "parking":
				Gear = 1;
				break;
			case "slow":
				Gear = 2;
				break;
			case "normal":
				Gear = 3;
				break;
			case "race":
				Gear = 4;
				break;
			default:
				Gear = 1;
				break;
		}	
		
		//Selecting mode
		switch (Storage) {
			case "":
				SelectMode(arg);
				break;
			default:
				SelectMode(Storage);
				break;
		}				

		if (UseLCD) UpdateLCD();
		if (debug) Echo("Storage at the end: "+Storage);
		if (arg == "debug") throw new Exception("MANUAL STOP");
	}
	current_tick++;
	if (!((arg == "") || (arg == "gearup") ||(arg == "geardown"))) Storage = arg;
	if (arg == "gearup" || arg == "geardown") Shift = arg;
		
}

//Speed calculations. Original code by Whiplash141 (from steam)
void PosVelAccel() {   
   
    if(!hasRun)   
    {   
        previousPos = Me.GetPosition();   
        previousVelocity = 0;   
        hasRun = true;   
    }   
    else   
    {            
    //---Find our current position   
        currentPos = Me.GetPosition();  
          
    //---Find our current velocity   
        delta_distance = Vector3D.Distance( currentPos , previousPos);   
        currentVelocity = Math.Round(delta_distance / ((double)tick_limit / 60) , 1);
		Velocity = currentVelocity;
		Velocity=Velocity/1000*3600;
		Velocity = Math.Round(Velocity, 1);
		
    //---Make current values our previous values   
        previousPos = currentPos;   
        previousVelocity = currentVelocity;   
    }   
} 

//Function to find block group by name
void GetGroups() {
	
	if (debug) Echo("Collecting groups");
	(GridTerminalSystem.GetBlockGroupWithName(GroupName)).GetBlocks(WheelsMain);
	if (debug) Echo("Main group done");
	if (RearSteer) {(GridTerminalSystem.GetBlockGroupWithName(GroupNameBack)).GetBlocks(GroupBack);}
	if (debug) Echo("Back group done");
	(GridTerminalSystem.GetBlockGroupWithName(GroupNameAll)).GetBlocks(GroupAll);
	if (debug) Echo("All group done");
	if (DownforceThrusters) {(GridTerminalSystem.GetBlockGroupWithName(ThrustersDownName)).GetBlocks(ThrustersDown);}
	if (debug) Echo("Thruster group done");
	
	if (!(GroupParkingName == "")) {(GridTerminalSystem.GetBlockGroupWithName(GroupParkingName)).GetBlocks(GroupParking);}
	if (debug) Echo("Parking group done");
	if (!(GroupSlowName == "")) {(GridTerminalSystem.GetBlockGroupWithName(GroupSlowName)).GetBlocks(GroupSlow);}
	if (debug) Echo("Slow group done");
	if (!(GroupNormalName == "")) {(GridTerminalSystem.GetBlockGroupWithName(GroupNormalName)).GetBlocks(GroupNormal);}
	if (debug) Echo("Normal group done");
	if (!(GroupRaceName == "")) {(GridTerminalSystem.GetBlockGroupWithName(GroupRaceName)).GetBlocks(GroupRace);}
	if (debug) Echo("Race group done");
	
	CustomGroupsAll.AddRange(GroupParking);
	CustomGroupsAll.AddRange(GroupSlow);
	CustomGroupsAll.AddRange(GroupNormal);
	CustomGroupsAll.AddRange(GroupRace);
	if (debug) Echo("Groups collected");
}

//Function to find a block with given name prefix.
IMyTerminalBlock get_block(string name){
	List<IMyTerminalBlock> blks = new List<IMyTerminalBlock>{};
	GridTerminalSystem.GetBlocksOfType<IMyTextPanel>(blks, block => block.CustomName.StartsWith(name));
	if (!blks[0].IsWorking ){
		throw new Exception(name + " Not Found or not working");     
    return null;
	}
	return blks[0];  
} 

//Gears control
void ApplyGear() {
	switch (Gear){
		case 1:
			Storage = "parking";
			SelectMode(Storage);
			break;
		case 2:
			Storage = "slow";
			SelectMode(Storage);
			break;
		case 3:
			Storage = "normal";
			SelectMode(Storage);
			break;
		case 4:
			Storage = "race";
			SelectMode(Storage);
			break;
		default:
			Storage = "parking";
			SelectMode(Storage);
			break;
	}
}
//To call if no argument provided
void NoArg() {
	if (Storage == "") {Echo("No argument given, storage is empty, loading default profile"); Storage = "normal"; SelectMode(Storage);}
	else {Echo("Automatic run detected. Stored profile: "+Storage); SelectMode(Storage);}
}


//Selecting drive mode according to arguments provided
void SelectMode(string mode){
	if (debug) Echo("Selecting mode");
	switch (Shift) {
		case "gearup":
			if (!(Gear > 3)) Gear = Gear + 1;
			if (debug) Echo("Applying gear shift UP from queue");
			Shift = "";
			ApplyGear();
			if (debug) Echo("Success");
			//FirstRun = true;
			return;
			break;
		case "geardown":
			if (!(Gear < 2)) Gear = Gear - 1;
			if (debug) Echo("Applying gear shift DOWN from queue");
			Shift = "";
			ApplyGear();
			if (debug) Echo("Success");
			//FirstRun = true;
			return;
			break;
	}
	switch (mode) {
	case "parking":
		if (debug) Echo("Switching to Parking mode");
		ApplySettings("parking", MaxSpeedParking, true, false, PowerParking, FrictionParking, DampingParking, StrengthParking, HeightParking, TravelParking);
		if (debug) Echo("Success");
		break;
	case "slow":
		if (debug) Echo("Switching to Slow mode");
		if (RearSteerOnSlow) {ApplySettings("slow", MaxSpeedSlow, true, false, PowerSlow, FrictionSlow, DampingSlow, StrengthSlow, HeightSlow, TravelSlow);}
		else {ApplySettings("slow", MaxSpeedSlow, false, false, PowerSlow, FrictionSlow, DampingSlow, StrengthSlow, HeightSlow, TravelSlow);}
		if (debug) Echo("Success");
		break;
	case "normal":
		if (debug) Echo("Switching to Normal mode");
		ApplySettings("normal", MaxSpeedNormal, false, false, PowerNormal, FrictionNormal, DampingNormal, StrengthNormal, HeightNormal, TravelNormal);
		if (debug) Echo("Success");
		break;
	case "race":
		if (debug) Echo("Switching to Race mode");
		ApplySettings("race", MaxSpeedRace, false, true, PowerRace, FrictionRace, DampingRace, StrengthRace, HeightRace, TravelRace);
		if (debug) Echo("Success");
		break;
	case "gearup":
		if (!(Gear > 3)) Gear = Gear + 1;
		if (debug) Echo("Applying gear shift UP from arg");
		ApplyGear();
		if (debug) Echo("Success");
		Shift = "";
		break;
	case "geardown":
		if (!(Gear < 2)) Gear = Gear - 1;
		if (debug) Echo("Applying gear shift DOWN from arg");
		ApplyGear();
		if (debug) Echo("Success");
		Shift = "";
		break;
	default:
		NoArg();
		break;
	}
}

//Applying settings, provided by mode selected
void ApplySettings(string Mode, string MaxSpeed, bool EnableRear, bool EnableThrusters, string Power, string Friction, string Damping, string Strength, string Height, string Travel){
	//Setting steering wheels max steer angle according to speed
	if (debug) Echo("Applying settings. Main steering");
    for (int i=0; i < WheelsMain.Count; i++) {
        TerminalPropertyExtensions.SetValueFloat(WheelsMain[i], "MaxSteerAngle", float.Parse(Angle));
    }

	//Rear steering settings
	if (debug) Echo("Applying settings. Rear steering");
	if (RearSteer) { //Will only work with RearSteer enabled
		for (int i=0; i < GroupBack.Count; i++) {
			IMyMotorSuspension Target = (IMyMotorSuspension)GroupBack[i];
			if ((EnableRear && !Target.Steering) || (!EnableRear && Target.Steering)) Target.GetActionWithName("Steering").Apply(Target); //Toggling rear steering if needed
			if (DynamicRearSteer) TerminalPropertyExtensions.SetValueFloat(Target, "MaxSteerAngle", float.Parse(RAngle)); //Setting rear wheels max steering angle if DynamicRearSteer enabled
		}
		
	}
	
	//Settings for All Wheels group.
	if (debug) Echo("Applying settings. Misc settings");
	for (int i=0; i < GroupAll.Count; i++) {
		if (!IgnoreSpeed) TerminalPropertyExtensions.SetValueFloat(GroupAll[i], "Speed Limit", float.Parse(MaxSpeed));
		if (!IgnorePower) TerminalPropertyExtensions.SetValueFloat(GroupAll[i], "Power", float.Parse(Power));
		if (!IgnoreFriction) TerminalPropertyExtensions.SetValueFloat(GroupAll[i], "Friction", float.Parse(Friction));
		//if (!IgnoreDamping) TerminalPropertyExtensions.SetValueFloat(GroupAll[i], "Damping", float.Parse(Damping));
		if (!IgnoreStrength) TerminalPropertyExtensions.SetValueFloat(GroupAll[i], "Strength", float.Parse(Strength));
		if (!IgnoreHeight) TerminalPropertyExtensions.SetValueFloat(GroupAll[i], "Height", float.Parse(Height));
		//if (!IgnoreTravel) TerminalPropertyExtensions.SetValueFloat(GroupAll[i], "Travel", float.Parse(Travel));
	}
	
	//Toggling thrusters if needed
	if (debug) Echo("Applying settings. Thrusters");
	if (DownforceThrusters) {
		for (int i=0; i < ThrustersDown.Count; i++) {
			if (EnableThrusters) {ThrustersDown[i].GetActionWithName("OnOff_On").Apply(ThrustersDown[i]);}
			else {ThrustersDown[i].GetActionWithName("OnOff_Off").Apply(ThrustersDown[i]);}
		}
	}
	
	//Toggling custom groups if enabled
	if (debug) Echo("Applying settings. Custom groups");
	switch (Mode) {
		case "parking":
			if (CustomGroupsAll.Count > 0) {
				for (int i=0; i < CustomGroupsAll.Count; i++) {
					CustomGroupsAll[i].GetActionWithName("OnOff_Off").Apply(CustomGroupsAll[i]);
				}
			}
			if (!(GroupParkingName == "")) {
				for (int i=0; i < GroupParking.Count; i++) {
					GroupParking[i].GetActionWithName("OnOff_On").Apply(GroupParking[i]);
				}
			}
			break;
		case "slow":
			if (CustomGroupsAll.Count > 0) {
				for (int i=0; i < CustomGroupsAll.Count; i++) {
					CustomGroupsAll[i].GetActionWithName("OnOff_Off").Apply(CustomGroupsAll[i]);
				}
			}
			if (!(GroupSlowName == "")) {
				for (int i=0; i < GroupSlow.Count; i++) {
					GroupSlow[i].GetActionWithName("OnOff_On").Apply(GroupSlow[i]);
				}
			}
			break;
		case "normal":
			if (CustomGroupsAll.Count > 0) {
				for (int i=0; i < CustomGroupsAll.Count; i++) {
					CustomGroupsAll[i].GetActionWithName("OnOff_Off").Apply(CustomGroupsAll[i]);
				}
			}
			if (!(GroupNormalName == "")) {
				for (int i=0; i < GroupNormal.Count; i++) {
					GroupNormal[i].GetActionWithName("OnOff_On").Apply(GroupNormal[i]);
				}
			}
			break;
		case "race":
			if (CustomGroupsAll.Count > 0) {
				for (int i=0; i < CustomGroupsAll.Count; i++) {
					CustomGroupsAll[i].GetActionWithName("OnOff_Off").Apply(CustomGroupsAll[i]);
				}
			}
			if (!(GroupRaceName == "")) {
				for (int i=0; i < GroupRace.Count; i++) {
					GroupRace[i].GetActionWithName("OnOff_On").Apply(GroupRace[i]);
				}
			}
			break;
	}
	
	if (debug) Echo("All settings applied");
	
}


//To call for information display
void UpdateLCD() {
	//Preparing to build text
	IMyTextPanel LCD = (IMyTextPanel)get_block(LCDNamePrefix);
	string text = "";
	string temp = "";
	
	//Building text
	if (debug) Echo("Building LCD text");
	text = "Vehicle status:\n\n";
	text = text+"Speed: "+Velocity.ToString()+"km/h.\n";
	text = text+"Steering angle: "+(Math.Round((Double.Parse(Angle)), 2)).ToString()+"deg.\n";
	text = text+"Current profile: "+Storage+".\n\n";
	if (RearSteer && (Storage == "parking" || (Storage == "slow" && RearSteerOnSlow))) {temp = "ON";}
	else {temp = "OFF";}
	text = text+"Rear steer: "+temp+".\n";
	if (DynamicRearSteer) temp = "ON";
	if (!DynamicRearSteer) temp = "OFF";
	text = text+"Rear dyn: "+temp+".\n";
	if (DownforceThrusters && Storage == "race") {temp = "ON";}
	else {temp = "OFF";}
	text = text+"Thrusters: "+temp+".\n";
	

	//Outputting to LCD
	if (debug) Echo("Outputting LCD text");
	LCD.ShowPublicTextOnScreen();
	LCD.WritePublicText(text);
}